import 'dart:async';
import 'package:awesome_dialog/awesome_dialog.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:modal_bottom_sheet/modal_bottom_sheet.dart';
import 'package:native_progress_hud/native_progress_hud.dart';
import 'package:simple_animations/simple_animations.dart';
import 'package:flutter/services.dart';
import 'package:intl/intl.dart';
import 'package:webview_flutter/webview_flutter.dart';

import '../payunitpackage.dart';

class PayButton extends StatefulWidget {
  final Function pressEvent;
  final String text;
  final IconData icon;
  final double width;
  final bool isFixedHeight;
  final Color color;
  final String transactionId;
  final String transonAmount;
  final String transactionCallBackUrl;
  final String redirectRouteName;
  final bool canIredirect;

  const PayButton({
    @required this.pressEvent,
    this.text,
    this.icon,
    this.color,
    this.isFixedHeight = true,
    this.transactionId,
    this.transonAmount,
    this.transactionCallBackUrl,
    this.redirectRouteName,
    this.canIredirect,
    this.width = double.infinity,
  });

  @override
  _PayButtonState createState() => _PayButtonState();
}

class _PayButtonState extends State<PayButton> with AnimationMixin {
  Animation<double> _scale;

  @override
  void initState() {
    super.initState();
    final curveAnimation = CurvedAnimation(
        parent: controller, curve: Curves.easeIn, reverseCurve: Curves.easeIn);
    _scale = Tween<double>(begin: 1, end: 0.9).animate(curveAnimation);
  }

  void _onTapDown(TapDownDetails details) {
    controller.play(duration: Duration(milliseconds: 150));
  }

  void _onTapUp(TapUpDetails details) {
    if (controller.isAnimating) {
      controller.addStatusListener((status) {
        if (status == AnimationStatus.completed)
          controller.playReverse(duration: Duration(milliseconds: 100));
      });
    } else
      controller.playReverse(duration: Duration(milliseconds: 100));
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        showMaterialModalBottomSheet(
          context: context,
          builder: (context, scrollController) => PayDialog(
            transactionId: widget.transactionId,
            transactionCallBackUrl: widget.transactionCallBackUrl,
            transonAmount: widget.transonAmount,
            canIredirect: widget.canIredirect,
            redirectRouteName: widget.redirectRouteName,
          ),
        );
      },
      onTapDown: _onTapDown,
      onTapUp: _onTapUp,
      onTapCancel: () {
        controller.playReverse(
          duration: Duration(milliseconds: 100),
        );
      },
      child: Transform.scale(
        scale: _scale.value,
        child: _animatedButtonUI,
      ),
    );
  }

  Widget get _animatedButtonUI => Container(
    height: widget.isFixedHeight ? 50.0 : null,
    width: widget.width,
    padding: const EdgeInsets.all(8.0),
    decoration: BoxDecoration(
        borderRadius: BorderRadius.all(Radius.circular(100)),
        color: widget.color ?? Theme.of(context).primaryColor),
    child: Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        widget.icon != null
            ? Padding(
          padding: const EdgeInsets.only(left: 4.0),
          child: Icon(
            widget.icon,
            color: Colors.white,
          ),
        )
            : SizedBox(),
        SizedBox(
          width: 5,
        ),
        Flexible(
          fit: FlexFit.loose,
          child: Text(
            '${widget.text}',
            // maxLines: 1,
            textAlign: TextAlign.center,
            style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
                fontSize: 14),
          ),
        ),
      ],
    ),
  );
}

class PayDialog extends StatefulWidget {
  final String redirectRouteName;
  final bool canIredirect;
  final String transactionId;
  final transonAmount;
  final transactionCallBackUrl;

  const PayDialog({
    this.transactionId,
    this.transonAmount,
    this.transactionCallBackUrl,
    this.redirectRouteName,
    this.canIredirect,
  });

  @override
  _PayDialogState createState() => _PayDialogState();
}

class _PayDialogState extends State<PayDialog> with AnimationMixin {
  Animation<double> _scale;

  //Make A get request to get All the Actual Providers
  ApiService api = new ApiService();
  Future providerFuture;

  @override
  void initState() {
    super.initState();
    final curveAnimation = CurvedAnimation(
        parent: controller, curve: Curves.easeIn, reverseCurve: Curves.easeIn);
    _scale = Tween<double>(begin: 1, end: 0.9).animate(curveAnimation);
    providerFuture = api.InitiatePayment(
        transactionAmount: widget.transonAmount,
        transactionCallBackUrl: widget.transactionCallBackUrl,
        transactionId: widget.transactionId,
        context: context);
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        height: MediaQuery.of(context).size.height / 2,
        child: Column(
          children: [
            Expanded(
                child: FutureBuilder(
                    future: providerFuture,
                    builder: (BuildContext context, AsyncSnapshot snapshot) {
                      if (snapshot.data == null) {
                        return Container(
                          child: Center(child: CircularProgressIndicator()),
                        );
                      } else {
                        if (snapshot.data.length <= 0) {
                          return Center(
                              child: Column(
                                mainAxisAlignment: MainAxisAlignment.center,
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: <Widget>[
                                  Container(
                                      height:
                                      MediaQuery.of(context).size.height * 0.15,
                                      child: Text("No Providers")),
                                  SizedBox(
                                    height:
                                    MediaQuery.of(context).size.height * 0.05,
                                  ),
                                ],
                              ));
                        } else {
                          return GridView.builder(
                            shrinkWrap: true,
                            itemCount: snapshot.data.length,
                            gridDelegate:
                            SliverGridDelegateWithFixedCrossAxisCount(
                                crossAxisCount: 3),
                            itemBuilder: (BuildContext context, int index) {
                              return GestureDetector(
                                onTap: () {
                                  String tag = snapshot.data[index]
                                  ['provider_short_tag'];
                                  print(" Tag : ${tag}");
                                  if (tag == 'mtnmomo' || tag == "orange") {
                                    Navigator.pop(context);
                                    showMaterialModalBottomSheet(
                                      context: context,
                                      builder: (context, scrollController) =>
                                          PayDialogWithMomo(
                                            transactionId: widget.transactionId,
                                            transactionAmount: widget.transonAmount,
                                            canIredirect: widget.canIredirect,
                                            redirectRouteName:
                                            widget.redirectRouteName,
                                            provider_short_tag: tag,
                                          ),
                                    );
                                  } else if (tag == 'eu') {
                                    Navigator.pop(context);
                                    showMaterialModalBottomSheet(
                                      context: context,
                                      builder: (context, scrollController) =>
                                          PayDialogWithExpressUnion(
                                            transactionId: widget.transactionId,
                                            transactionAmount: widget.transonAmount,
                                            canIredirect: widget.canIredirect,
                                            redirectRouteName:
                                            widget.redirectRouteName,
                                            provider_short_tag: snapshot.data[index]
                                            ['provider_short_tag'],
                                          ),
                                    );
                                  } else if (tag == 'ecobank') {
                                    Navigator.pop(context);
                                    showMaterialModalBottomSheet(
                                        context: context,
                                        builder: (context, scrollController) =>
                                            PayDialogWithEcobank(
                                              transactionId:
                                              widget.transactionId,
                                              transactionAmount:
                                              widget.transonAmount,
                                              canIredirect: widget.canIredirect,
                                              redirectRouteName:
                                              widget.redirectRouteName,
                                              provider_short_tag: tag,
                                            ));
                                  }
                                },
                                child: Container(
                                  margin: EdgeInsets.zero,
                                  padding: EdgeInsets.only(left: 5, right: 5),
                                  child: Image(
                                    image: NetworkImage(
                                      snapshot.data[index]['provider_logo'],
                                    ),
                                    fit: BoxFit.fitWidth,
                                  ),
                                ),
                              );
                            },
                          );
                        }
                      }
                    })),
            Text(
              "Choice Your Payment method.",
              style:
              TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
            ),
            SizedBox(
              height: MediaQuery.of(context).size.height * 0.06,
            ),
          ],
        ));
  }
}

class PayDialogWithMomo extends StatefulWidget {
  final String transactionId;
  final String transactionAmount;
  final String redirectRouteName;
  final bool canIredirect;
  final String provider_short_tag;
  final String phoneNumber;
  final String transactionCallBackUrl;

  const PayDialogWithMomo(
      {this.transactionId,
        this.transactionAmount,
        this.redirectRouteName,
        this.canIredirect,
        this.provider_short_tag,
        this.transactionCallBackUrl,
        this.phoneNumber});

  @override
  _PayDialogWithTelcoState createState() => _PayDialogWithTelcoState();
}

class _PayDialogWithTelcoState extends State<PayDialogWithMomo>
    with AnimationMixin {
  final phoneController = TextEditingController();

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      child: Container(
        height: MediaQuery.of(context).size.height / 1.5,
        child: Padding(
          padding: EdgeInsets.only(left: 30, right: 30),
          child: Column(
            children: <Widget>[
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.05,
              ),
              Row(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Text(
                    "Transaction ID :",
                    style: TextStyle(
                        color: Colors.black,
                        fontSize: MediaQuery.of(context).size.width * 0.040,
                        fontWeight: FontWeight.bold),
                  ),
                  SizedBox(
                    width: MediaQuery.of(context).size.height * 0.01,
                  ),
                  Flexible(
                    child: Text(
                      "${widget.transactionId}",
                      style: TextStyle(
                          color: Colors.black,
                          fontSize: MediaQuery.of(context).size.width * 0.040,
                          fontWeight: FontWeight.bold),
                      overflow: TextOverflow.ellipsis,
                      maxLines: 1,
                    ),
                  )
                ],
              ),
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.01,
              ),
              Text(
                "${getOcy().format((int.parse(widget.transactionAmount)))} XAF",
                style: TextStyle(
                    color: Colors.green,
                    fontSize: MediaQuery.of(context).size.width * 0.080,
                    fontWeight: FontWeight.bold),
              ),
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.01,
              ),
              Text(
                "Enter your phone number",
                style: TextStyle(
                    color: Colors.black,
                    fontSize: MediaQuery.of(context).size.width * 0.045,
                    fontWeight: FontWeight.bold),
              ),
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.03,
              ),
              TextField(
                keyboardType: TextInputType.number,
                controller: phoneController,
                decoration: InputDecoration(
                  contentPadding: EdgeInsets.all(
                      MediaQuery.of(context).size.height * 0.020),
                  hintText: "Enter your phone number",
                  hintStyle: TextStyle(
                    color: Colors.grey,
                    fontWeight: FontWeight.bold,
                    fontSize: MediaQuery.of(context).size.width * 0.032,
                  ),
                  border: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(50.0),
                  ),
                  prefixIcon: Icon(
                    FontAwesomeIcons.phoneAlt,
                    size: 16,
                  ),
                ),
              ),
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.05,
              ),
              AnimButton(
                color: Colors.orange,
                pressEvent: () {
                  String phoneNumber = phoneController.text;
                  if (phoneNumber.isEmpty) {
                    ///show diaolog
                    print('Phone number is empty');
                  } else {
                    AwesomeDialog(
                        dismissOnBackKeyPress: false,
                        dismissOnTouchOutside: false,
                        context: context,
                        dialogType: DialogType.INFO,
                        animType: AnimType.BOTTOMSLIDE,
                        title: "Confirm",
                        desc: 'Do you confirm this payment',
                        btnOkColor: Colors.green,
                        btnOkText: "Confirm",
                        btnOkIcon: Icons.check,
                        btnOkOnPress: () {
                          print('redirected : ${widget.canIredirect}');
                          ApiService api = new ApiService();
                          api.makePayment(
                              transactionId: widget.transactionId,
                              transactionAmount: widget.transactionAmount,
                              transactionCallBackUrl:
                              widget.transactionCallBackUrl,
                              phoneNumber: phoneController.text,
                              provider_short_tag: widget.provider_short_tag);
                        },
                        btnCancelOnPress: () {},
                        btnCancelColor: Colors.red,
                        btnCancelIcon: Icons.cancel,
                        btnCancelText: 'Cancel')
                      ..show();
                  }
                },
                text: 'Make Payment',
              )
            ],
          ),
        ),
      ),
    );
  }

  ////////////////////utility class
  progressHub(msg, context) {
    NativeProgressHud.showWaitingWithText(msg);
  }

  closeDialog(context) {
    NativeProgressHud.hideWaiting();
  }

  NumberFormat getOcy() {
    final oCcy = new NumberFormat("#,##0", "en_us");
    return oCcy;
  }
}

//////////////
class PayDialogWithExpressUnion extends StatefulWidget {
  final String transactionId;
  final String transactionAmount;
  final String redirectRouteName;
  final bool canIredirect;
  final String provider_short_tag;
  final String phoneNumber;
  final String transactionCallBackUrl;

  const PayDialogWithExpressUnion(
      {this.transactionId,
        this.transactionAmount,
        this.redirectRouteName,
        this.canIredirect,
        this.provider_short_tag,
        this.transactionCallBackUrl,
        this.phoneNumber});

  @override
  _PayDialogWithExpressUnionState createState() =>
      _PayDialogWithExpressUnionState();
}

class _PayDialogWithExpressUnionState extends State<PayDialogWithExpressUnion>
    with AnimationMixin {
  final phoneController = TextEditingController();
  final userNameController = TextEditingController();

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      child: Container(
        height: MediaQuery.of(context).size.height / 1.5,
        child: Padding(
          padding: EdgeInsets.only(left: 30, right: 30),
          child: Column(
            children: <Widget>[
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.05,
              ),
              Row(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Text(
                    "Transaction ID :",
                    style: TextStyle(
                        color: Colors.black,
                        fontSize: MediaQuery.of(context).size.width * 0.040,
                        fontWeight: FontWeight.bold),
                  ),
                  SizedBox(
                    width: MediaQuery.of(context).size.height * 0.01,
                  ),
                  Flexible(
                    child: Text(
                      "${widget.transactionId}",
                      style: TextStyle(
                          color: Colors.black,
                          fontSize: MediaQuery.of(context).size.width * 0.040,
                          fontWeight: FontWeight.bold),
                      overflow: TextOverflow.ellipsis,
                      maxLines: 1,
                    ),
                  )
                ],
              ),
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.01,
              ),
              Text(
                "${getOcy().format((int.parse(widget.transactionAmount)))} XAF",
                style: TextStyle(
                    color: Colors.green,
                    fontSize: MediaQuery.of(context).size.width * 0.080,
                    fontWeight: FontWeight.bold),
              ),
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.01,
              ),
//              Text(
//                "Enter your Express Union C",
//                style: TextStyle(
//                    color: Colors.black,
//                    fontSize: MediaQuery.of(context).size.width * 0.045,
//                    fontWeight: FontWeight.bold),
//              ),
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.05,
              ),
              TextField(
                keyboardType: TextInputType.text,
                controller: userNameController,
                decoration: InputDecoration(
                  contentPadding: EdgeInsets.all(
                      MediaQuery.of(context).size.height * 0.020),
                  hintText: "Enter your User name",
                  hintStyle: TextStyle(
                    color: Colors.grey,
                    fontWeight: FontWeight.bold,
                    fontSize: MediaQuery.of(context).size.width * 0.032,
                  ),
                  border: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(50.0),
                  ),
                  prefixIcon: Icon(
                    FontAwesomeIcons.userCircle,
                    size: 16,
                  ),
                ),
              ),

              SizedBox(
                height: MediaQuery.of(context).size.height * 0.03,
              ),
              TextField(
                keyboardType: TextInputType.number,
                controller: phoneController,
                decoration: InputDecoration(
                  contentPadding: EdgeInsets.all(
                      MediaQuery.of(context).size.height * 0.020),
                  hintText: "Enter your phone number",
                  hintStyle: TextStyle(
                    color: Colors.grey,
                    fontWeight: FontWeight.bold,
                    fontSize: MediaQuery.of(context).size.width * 0.032,
                  ),
                  border: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(50.0),
                  ),
                  prefixIcon: Icon(
                    FontAwesomeIcons.phoneAlt,
                    size: 16,
                  ),
                ),
              ),
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.05,
              ),
              AnimButton(
                color: Colors.orange,
                pressEvent: () {
                  String phoneNumber = phoneController.text;
                  String userName = userNameController.text;
                  if (phoneNumber.isEmpty || userName.isEmpty) {
                    ///show diaolog alert the field is empty
                    print('Phone number is empty');
                  } else {
                    AwesomeDialog(
                        dismissOnBackKeyPress: false,
                        dismissOnTouchOutside: false,
                        context: context,
                        dialogType: DialogType.INFO,
                        animType: AnimType.BOTTOMSLIDE,
                        title: "Confirm",
                        desc: 'Do you want to confirm this payment',
                        btnOkColor: Colors.green,
                        btnOkText: "Confirm",
                        btnOkIcon: Icons.check,
                        btnOkOnPress: () {
                          print('redirected : ${widget.canIredirect}');
                          ApiService api = new ApiService();
                          api.makePayment(
                            transactionId: widget.transactionId,
                            transactionAmount: widget.transactionAmount,
                            transactionCallBackUrl:
                            widget.transactionCallBackUrl,
                            phoneNumber: phoneController.text,
                            provider_short_tag: widget.provider_short_tag,
                            userName: userName,
                          );
                        },
                        btnCancelOnPress: () {},
                        btnCancelColor: Colors.red,
                        btnCancelIcon: Icons.cancel,
                        btnCancelText: 'Cancel')
                      ..show();
                  }
                },
                text: 'Make Payment',
              )
            ],
          ),
        ),
      ),
    );
  }
}

//////////////
class PayDialogWithEcobank extends StatefulWidget {
  final String transactionId;
  final String transactionAmount;
  final String redirectRouteName;
  final bool canIredirect;
  final String provider_short_tag;
  final String phoneNumber;
  final String transactionCallBackUrl;

  const PayDialogWithEcobank(
      {this.transactionId,
        this.transactionAmount,
        this.redirectRouteName,
        this.canIredirect,
        this.provider_short_tag,
        this.transactionCallBackUrl,
        this.phoneNumber});

  @override
  _PayDialogWithEcobankState createState() => _PayDialogWithEcobankState();
}

class _PayDialogWithEcobankState extends State<PayDialogWithEcobank>
    with AnimationMixin {
  final Completer<WebViewController> _controller =
  Completer<WebViewController>();

  @override
  void initState() {
    super.initState();
  }

  final Set<Factory> gestureRecognizers = [
    Factory(() => EagerGestureRecognizer()),
  ].toSet();

  @override
  Widget build(BuildContext context) {
    return Container(
      height: MediaQuery.of(context).size.height / 1.5,
      child: WebView(
          initialUrl: 'https://flutter.dev',
          javascriptMode: JavascriptMode.unrestricted,
          onWebViewCreated: (WebViewController webViewController) {
            _controller.complete(webViewController);
          },
          gestureNavigationEnabled: true,
          gestureRecognizers: gestureRecognizers),
    );
  }
}

////////////// Utils
progressHub(msg, context) {
  NativeProgressHud.showWaitingWithText(msg);
}

closeDialog(context) {
  NativeProgressHud.hideWaiting();
}

NumberFormat getOcy() {
  final oCcy = new NumberFormat("#,##0", "en_us");
  return oCcy;
}

class AnimButton extends StatefulWidget {
  final Function pressEvent;
  final String text;
  final IconData icon;
  final double width;
  final bool isFixedHeight;
  final Color color;

  const AnimButton(
      {@required this.pressEvent,
        this.text,
        this.icon,
        this.color,
        this.isFixedHeight = true,
        this.width = double.infinity});

  @override
  _AnimatedButtonState createState() => _AnimatedButtonState();
}

class _AnimatedButtonState extends State<AnimButton> with AnimationMixin {
  Animation<double> _scale;

  @override
  void initState() {
    super.initState();
    final curveAnimation = CurvedAnimation(
        parent: controller, curve: Curves.easeIn, reverseCurve: Curves.easeIn);
    _scale = Tween<double>(begin: 1, end: 0.9).animate(curveAnimation);
  }

  void _onTapDown(TapDownDetails details) {
    controller.play(duration: Duration(milliseconds: 150));
  }

  void _onTapUp(TapUpDetails details) {
    if (controller.isAnimating) {
      controller.addStatusListener((status) {
        if (status == AnimationStatus.completed)
          controller.playReverse(duration: Duration(milliseconds: 100));
      });
    } else
      controller.playReverse(duration: Duration(milliseconds: 100));
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        widget.pressEvent();
        //  _controller.forward();
      },
      onTapDown: _onTapDown,
      onTapUp: _onTapUp,
      onTapCancel: () {
        controller.playReverse(
          duration: Duration(milliseconds: 100),
        );
      },
      child: Transform.scale(
        scale: _scale.value,
        child: _animatedButtonUI,
      ),
    );
  }

  Widget get _animatedButtonUI => Container(
    height: widget.isFixedHeight ? 50.0 : null,
    width: widget.width,
    padding: const EdgeInsets.all(8.0),
    decoration: BoxDecoration(
        borderRadius: BorderRadius.all(Radius.circular(100)),
        color: widget.color ?? Theme.of(context).primaryColor),
    child: Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        widget.icon != null
            ? Padding(
          padding: const EdgeInsets.only(left: 4.0),
          child: Icon(
            widget.icon,
            color: Colors.white,
          ),
        )
            : SizedBox(),
        SizedBox(
          width: 5,
        ),
        Flexible(
          fit: FlexFit.loose,
          child: Text(
            '${widget.text}',
            // maxLines: 1,
            textAlign: TextAlign.center,
            style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
                fontSize: 14),
          ),
        ),
      ],
    ),
  );
}
